/*-------------Astrol.Component.RichText----------------*/

export function CreateQuill(quillElement, toolBar, readOnly, placeholder, theme, formats, debugLevel) {
    Quill.register('modules/blotFormatter', QuillBlotFormatter.default);

    var options = {
        debug: debugLevel,
        modules: {
            toolbar: toolBar,
            blotFormatter: {}
        },
        placeholder: placeholder,
        readOnly: readOnly,
        theme: theme
    };

    if (formats) {
        options.formats = formats;
    }

    new Quill(quillElement, options);
}

export function GetQuillContent(quillElement) {
    return JSON.stringify(quillElement.__quill.getContents());
}

export function GetQuillText(quillElement) {
    return quillElement.__quill.getText();
}

export function GetQuillHTML(quillElement) {
    return quillElement.__quill.root.innerHTML;
}

export function LoadQuillContent(quillElement, quillContent) {
    var content = JSON.parse(quillContent);
    return quillElement.__quill.setContents(content, 'api');
}

export function LoadQuillHTMLContent(quillElement, quillHTMLContent) {
    return quillElement.__quill.root.innerHTML = quillHTMLContent;
}

export function EnableQuillEditor(quillElement, mode) {
    quillElement.__quill.enable(mode);
}

export function InsertQuillImage(quillElement, imageURL) {
    var Delta = Quill.import('delta');
    var editorIndex = 0;

    if (quillElement.__quill.getSelection() !== null) {
        editorIndex = quillElement.__quill.getSelection().index;
    }

    return quillElement.__quill.updateContents(
        new Delta()
            .retain(editorIndex)
            .insert({ image: imageURL },
                { alt: imageURL }));
}

export function InsertQuillVideo(quillElement, videoURL) {
    var Delta = Quill.import('delta');
    var editorIndex = 0;

    if (quillElement.__quill.getSelection() !== null) {
        editorIndex = quillElement.__quill.getSelection().index;
    }

    return quillElement.__quill.updateContents(
        new Delta()
            .retain(editorIndex)
            .insert({ video: videoURL },
                { alt: videoURL }));
}

export function InsertQuillText(quillElement, text) {
    var editorIndex = 0;
    var selectionLength = 0;

    if (quillElement.__quill.getSelection() !== null) {
        var selection = quillElement.__quill.getSelection();
        editorIndex = selection.index;
        selectionLength = selection.length;
    }

    return quillElement.__quill.deleteText(editorIndex, selectionLength)
        .concat(quillElement.__quill.insertText(editorIndex, text));
}

/*-------------Astrol.Component.FileExplorer----------------*/
export function RemoveAllOcurrencesSelectorID(selector) {
    var items = document.querySelectorAll(selector);
    if (items.length > 0) {
        items.forEach(function (item) {
            var clases = item.getAttribute('id');
            clases = clases.replace(item.id, '');
            item.setAttribute('id', clases.trim());
        });
    }
}

export function AddIdByClase(itemClass, newId) {
    var elementos = document.getElementsByClassName(itemClass);
    if (elementos.length > 0) {
        for (var i = 0; i < elementos.length; i++) {
            elementos[i].id = newId;
        }
    }
}
/*----------------Astrol.Component.Basic-------------------*/
export function CountElementExistToHTML(selector) {
    var element = document.querySelector(selector);
    if (element) {
        return document.querySelectorAll(selector).length;
    }
    return 0;
}

export function FocusElement(selector) {
    var element = document.querySelector(selector);
    if (element) {
        element.focus();
    }
}

export function MaxHeight(id) {
    var height = window.innerHeight;
    var mConten = document.getElementById(id);
    mConten.style.cssText = 'max-height: ' + (height - 125) + 'px; overflow: auto;';
}

export function DragAndDropModal(clasName) {
    var modal = document.querySelector(clasName);
    //var modalHeader = document.querySelector(clasName + " .head-modal");

    interact(modal)
        .draggable({
            // Filtra los elementos arrastrables a la cabecera
            ignoreFrom: '.body-astrol-modal, .button-astrol-modal',
            // Restringe el movimiento a los ejes X e Y
            restrict: {
                restriction: 'parent',
                endOnly: true,
                elementRect: { top: 0, left: 0, bottom: 1, right: 1 }
            }
        })
        .on('dragmove', function (event) {
            var target = event.target;
            var x = (parseFloat(target.getAttribute('data-x')) || 0) + event.dx;
            var y = (parseFloat(target.getAttribute('data-y')) || 0) + event.dy;

            // Actualiza la posicin del modal al arrastrarlo
            target.style.transform = 'translate(' + x + 'px, ' + y + 'px)';
            target.setAttribute('data-x', x);
            target.setAttribute('data-y', y);
        });
}

export function DragAndDropConfirm(clasName) {
    const position = { x: 0, y: 0 }

    interact(clasName).draggable({
        listeners: {
            //start(event) {
            //    console.log(event.type, event.target)
            //},
            move(event) {
                position.x += event.dx
                position.y += event.dy

                event.target.style.transform =
                    `translate(${position.x}px, ${position.y}px)`
            },
            restrict: {
                restriction: 'parent',
                endOnly: true,
                elementRect: { top: 0, left: 0, bottom: 1, right: 1 }
            }
        }
    });
}

export function Resizable(clasName) {
    interact(clasName)
        .resizable({
            edges: { top: true, left: true, bottom: true, right: true },
            listeners: {
                move: function (event) {
                    let { x, y } = event.target.dataset

                    x = (parseFloat(x) || 0) + event.deltaRect.left
                    y = (parseFloat(y) || 0) + event.deltaRect.top

                    Object.assign(event.target.style, {
                        width: `${event.rect.width}px`,
                        height: `${event.rect.height}px`,
                        transform: `translate(${x}px, ${y}px)`
                    })

                    Object.assign(event.target.dataset, { x, y })
                }
            }
        });
}

export function ShowNotification(message = "", icon = "astrol-warning", showLoad = true, time = 5, style = "") {
    if (ExistContNotification()) {
        var contNotification = document.getElementById("cont-astrol-all-notification");
        if (contNotification !== null) {
            var id = GenerateUUID();
            var loadComponent = "";
            if (showLoad) {
                loadComponent =
                    "   <div class='cont-life-time-astrol-notification' >" +
                    "       <div class='life-time-astrol-notification' style='animation: " + time + "s time-notification;'></div > " +
                    "   </div > ";
            }
            var newItem =
                "<div id='" + id + "' class='cont-astrol-notification " + style + "' style='animation: " + time + "s notification;'>" +
                "   <div class='body-astrol-notification' >" +
                "       <i class='" + icon + "' ></i>" +
                "       <span>" + message + "</span>" +
                "   </div>" +
                loadComponent +
                "</div>";

            contNotification.insertAdjacentHTML("afterbegin", newItem);

            var time = ((time * 1000) + 100);
            setTimeout(LifeNotification, time, id);
        }
    }
}

function LifeNotification(id) {
    var notification = document.getElementById(id);
    if (notification !== null) {
        notification.remove();
    }
}

function GenerateUUID() {
    var d = new Date().getTime();//Timestamp
    var d2 = ((typeof performance !== 'undefined') && performance.now && (performance.now() * 1000)) || 0;//Time in microseconds since page-load or 0 if unsupported
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = Math.random() * 16;//random number between 0 and 16
        if (d > 0) {//Use timestamp until depleted
            r = (d + r) % 16 | 0;
            d = Math.floor(d / 16);
        } else {//Use microseconds since page-load if supported
            r = (d2 + r) % 16 | 0;
            d2 = Math.floor(d2 / 16);
        }
        return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
}

export function ExistContNotification() {
    var contNotification = document.getElementById("cont-astrol-all-notification");
    if (contNotification !== null) {//El contenedor SI existe 
        return true;
    }
    else {//El contenedor NO existe y lo creo

        const div = document.createElement("div");
        div.setAttribute("id", "cont-astrol-all-notification");
        document.body.insertAdjacentElement("afterbegin", div);
        return ExistContNotification();
    }
}

export function SetClassBySelector(selector, newClass) {
    const elements = document.querySelectorAll(selector);
    if (elements.length > 0) {
        elements.forEach(element => {
            element.className = newClass;
        });
    }
}

export function EnableNavButton(element, enable) {
    try {
        if (element != null) {
            var childAnchor = element.querySelector('a');
            if (enable) {
                element.classList.remove('disable-astrol');
                if (childAnchor != null) {
                    childAnchor.removeAttribute('tabindex');
                }
            } else {
                element.classList.add('disable-astrol');
                if (childAnchor != null) {
                    childAnchor.setAttribute('tabindex', '-1');
                }
            }
        }
    } catch { }
}

export function SetInputValue(element, value) {
    element.value = value;
}

export function IdleTimer(dotnetHelper, idleTime = 1800000 /* 30 minutos */) {
    let timerId;
    let lastActiveTime = Date.now();
    const resetTimer = () => {
        clearInterval(timerId);
        timerId = setInterval(checkIdleTime, 60000); // Verifica el tiempo de inactividad cada 1 minuto
    };

    const checkIdleTime = () => {
        const elapsedTime = Date.now() - lastActiveTime;
        if (elapsedTime >= idleTime) {
            clearInterval(timerId);
            logout();
        }
    };

    const logout = () => {
        dotnetHelper.invokeMethodAsync("Logout");
    };

    const observer = new IntersectionObserver(
        ([entry]) => {
            if (entry.isIntersecting) {
                lastActiveTime = Date.now();
                resetTimer();
            }
        },
        {
            root: null,
            threshold: 0
        }
    );

    observer.observe(document.documentElement);

    // Agrega listeners para varios eventos del DOM
    const eventListeners = ["mousemove", "keydown", "scroll", "touchstart", "touchmove", "touchend", "resize"];

    eventListeners.forEach((eventName) => {
        document.addEventListener(eventName, () => {
            lastActiveTime = Date.now();
            resetTimer();
        });
    });

    // Agrega una funcin para detener el temporizador en caso de que se deba cancelar la sesin
    const stopTimer = () => {
        clearInterval(timerId);
        observer.unobserve(document.documentElement);

        // Remueve listeners de eventos del DOM
        eventListeners.forEach((eventName) => {
            document.removeEventListener(eventName, resetTimer);
        });
    };

    resetTimer();

    return {
        stopTimer
    };
}

export function OnPaste(dropZone, inputFile) {
    dropZone.addEventListener('paste', onPaste);

    function onPaste(event) {
        inputFile.files = event.clipboardData.files;
        const changeEvent = new Event('change', { bubbles: true });
        inputFile.dispatchEvent(changeEvent);
    }

    return {
        dispose: () => {
            dropZone.removeEventListener('paste', onPaste);
        }
    }
}

export function saveAsFile(filename, bytesBase64) {
    var link = document.createElement('a');
    link.download = filename;
    link.href = "data:application/octet-stream;base64," + bytesBase64;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
}

export function GetWidthBySelector(selector) {
    const elemento = document.querySelector(selector);
    if (elemento) {
        return elemento.offsetWidth;
    } else {
        return 0;
    }
}

export function GetHeightBySelector(selector) {
    const elemento = document.querySelector(selector);
    if (elemento) {
        return elemento.offsetHeight;
    } else {
        return 0;
    }
}

export function CopyToClipboard(text) {
    try {
        const textArea = document.createElement('textarea');
        textArea.value = text;
        document.body.appendChild(textArea);
        textArea.select();
        document.execCommand('copy');
        document.body.removeChild(textArea);
        return true;
    } catch {
        return false;
    }
}

//export function saveAsFile(filename, bytes) {
//    const blob = new Blob([bytes], { type: 'application/octet-stream' });
//    const url = URL.createObjectURL(blob);
//    const link = document.createElement('a');
//    link.href = url;
//    link.download = filename;
//    link.click();
//    URL.revokeObjectURL(url);
//}

//export function GetHeightBySelector(selector) {
//    var item = document.querySelector(selector);
//    if (item) {
//        return item.offsetHeight;
//    }
//    return 0;
//}

//export function GetWidthBySelector(selector) {
//    var item = document.querySelector(selector);
//    if (item) {
//        return item.offsetWidth;
//    }
//    return 0;
//}